import 'dart:io' show Platform;

import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:info_tren/components/select_train_suggestions/select_train_suggestions.dart';
import 'package:info_tren/models/ui_design.dart';
import 'package:info_tren/pages/train_info_page/select_train/select_train_cupertino.dart';
import 'package:info_tren/pages/train_info_page/select_train/select_train_material.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info.dart';
import 'package:info_tren/utils/default_ui_design.dart';
import 'package:tuple/tuple.dart';

typedef TrainSelectedCallback(int trainNumber);

class SelectTrainPage extends StatefulWidget {
  final UiDesign? uiDesign;

  SelectTrainPage({Key? key, this.uiDesign}) : super(key: key);

  static String routeName = "/trainInfo/selectTrain";

  void onTrainSelected(BuildContext context, String selection) {
    Navigator.of(context).pushNamed(TrainInfo.routeName, arguments: selection);
  }

  @override
  SelectTrainPageState createState() {
    final uiDesign = this.uiDesign ?? defaultUiDesign;
    switch(uiDesign) {
      case UiDesign.MATERIAL:
        return SelectTrainPageStateMaterial();
      case UiDesign.CUPERTINO:
        return SelectTrainPageStateCupertino();
      default:
        throw UnmatchedUiDesignException(uiDesign);
    }
  }
}

abstract class SelectTrainPageState extends State<SelectTrainPage> {
  final String pageTitle = 'Informații despre tren';
  final String textFieldLabel = 'Numărul trenului';

  TextEditingController trainNoController = TextEditingController();

  @override
  void initState() {
    super.initState();
  }

  void onTextChanged() {
    setState(() {});
  }

  Widget get suggestionsList => SelectTrainSuggestions(
    uiDesign: widget.uiDesign, 
    userInput: trainNoController.text, 
    onTrainSelected: (trainNumber) => widget.onTrainSelected(context, trainNumber),
    key: ValueKey(trainNoController.text),
  );
}
