
import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:info_tren/api/train_data.dart';
import 'package:info_tren/components/loading/loading.dart';
import 'package:info_tren/components/refresh_future_builder.dart';
import 'package:info_tren/models/train_data.dart';
import 'package:info_tren/models/ui_design.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info_cupertino.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info_material.dart';
import 'package:info_tren/utils/default_ui_design.dart';


class TrainInfo extends StatelessWidget {
  static String routeName = "/trainInfo/display";

  final UiDesign? uiDesign;
  final String trainNumber;

  TrainInfo({Key? key, required this.trainNumber, this.uiDesign}): super(key: key);

  @override
  Widget build(BuildContext context) {
    final uiDesign = this.uiDesign ?? defaultUiDesign;

    return RefreshFutureBuilder<TrainData>(
      futureCreator: () => getTrain(trainNumber),
      builder: (context, refresh, snapshot) {

        switch (uiDesign) {
          case UiDesign.MATERIAL:
            if ([RefreshFutureBuilderState.none, RefreshFutureBuilderState.waiting].contains(snapshot.state)) {
              return TrainInfoLoadingMaterial(title: trainNumber.toString(), loadingText: "Se încarcă...",);
            }
            else if (snapshot.state == RefreshFutureBuilderState.error) {
              return TrainInfoErrorMaterial(title: '$trainNumber - Error', error: snapshot.error!, refresh: refresh,);
            }

            return TrainInfoMaterial(trainData: snapshot.data!, refresh: refresh,);
          case UiDesign.CUPERTINO:
            if ([RefreshFutureBuilderState.none, RefreshFutureBuilderState.waiting].contains(snapshot.state)) {
              return TrainInfoLoadingCupertino(title: trainNumber.toString(), loadingText: "Se încarcă...",);
            }
            else if (snapshot.state == RefreshFutureBuilderState.error) {
              return TrainInfoErrorCupertino(title: '$trainNumber - Error', error: snapshot.error!, refresh: refresh,);
            }

            return TrainInfoCupertino(trainData: snapshot.data!, refresh: refresh, isRefreshing: snapshot.state == RefreshFutureBuilderState.refreshing,);
          default:
            throw UnmatchedUiDesignException(uiDesign);
        }
      },
    );
  }
}

abstract class TrainInfoLoading extends StatelessWidget {
  final String title;
  final Widget loadingWidget;

  TrainInfoLoading({required this.title, String? loadingText, UiDesign? uiDesign}) : loadingWidget = Loading(uiDesign: uiDesign, text: loadingText,);
}

abstract class TrainInfoError extends StatelessWidget {
  final String title;
  final Object error;
  final Future Function()? refresh;

  TrainInfoError({required this.title, required this.error, this.refresh});
}
