import 'package:flutter/cupertino.dart';
import 'package:info_tren/models/train_data.dart';
import 'package:info_tren/pages/train_info_page/train_info_constants.dart';

class DisplayTrainStation extends StatelessWidget {
  final Station station;

  DisplayTrainStation({required this.station});

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Row(
          mainAxisSize: MainAxisSize.max,
          children: <Widget>[
            Builder(
              builder: (context) {
                final departureStatus = station.departure?.status;
                final arrivalStatus = station.arrival?.status;
                int delay;
                bool real;
                if (departureStatus == null) {
                  delay = arrivalStatus?.delay ?? 0;
                  real = arrivalStatus?.real ?? false;
                }
                else if (arrivalStatus == null) {
                  delay = departureStatus.delay;
                  real = departureStatus.real;
                }
                else {
                  delay = departureStatus.delay;
                  real = departureStatus.real;
                  if (!real && arrivalStatus.real) {
                    delay = arrivalStatus.delay;
                    real = arrivalStatus.real;
                  }
                }

                final isDelayed = delay > 0 && real == true;
                final isOnTime = delay <= 0 && real == true;
                final isNotScheduled = false;

                return Badge(
                  text: station.km.toString(),
                  caption: 'km',
                  isNotScheduled: isNotScheduled,
                  isDelayed: isDelayed,
                  isOnTime: isOnTime,
                );
              }
            ),
            Expanded(
              child: Title(
                station: station,
              ),
            ),
            if (station.platform == null)
              Container(width: 48, height: 48,)
            else
              Badge(text: station.platform!, caption: 'linia'),
          ],
        ),
        Time(
          station: station,
        ),
        Delay(
          station: station,
        ),
      ],
    );
  }
}

class Badge extends StatelessWidget {
  final String text;
  final String caption;
  final bool isNotScheduled;
  final bool isOnTime;
  final bool isDelayed;

  Badge({
    required this.text,
    required this.caption,
    this.isNotScheduled = false,
    this.isOnTime = false,
    this.isDelayed = false,
  });

  @override
  Widget build(BuildContext context) {
    Color foregroundColor = FOREGROUND_WHITE;
    Color? backgroundColor;

    if (isNotScheduled) {
      foregroundColor = Color.fromRGBO(225, 175, 30, 1);
      backgroundColor = Color.fromRGBO(80, 40, 10, 1);
    }
    else if (isOnTime) {
      foregroundColor = Color.fromRGBO(130, 175, 65, 1);
      backgroundColor = Color.fromRGBO(40, 80, 10, 1);
    }
    else if (isDelayed) {
      foregroundColor = Color.fromRGBO(225, 75, 30, 1);
      backgroundColor = Color.fromRGBO(80, 20, 10, 1);
    }

    return Padding(
      padding: const EdgeInsets.all(8),
      child: Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(10),
          border: Border.all(
            width: 2,
            color: foregroundColor,
          ),
          color: backgroundColor,
          // color: CupertinoColors.activeOrange,
        ),
        width: 48,
        height: 48,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Expanded(
              child: Center(
                child:  Text(
                  text,
                  style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                    fontSize: 20,
                    fontWeight: MediaQuery.of(context).boldText ? FontWeight.w400 : FontWeight.w200,
                    color: MediaQuery.of(context).boldText ? FOREGROUND_WHITE : foregroundColor,
                  ),
                  textAlign: TextAlign.center,
                ),
              ),
            ),
            Text(
              caption,
              style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                fontSize: 12,
                color: MediaQuery.of(context).boldText ? FOREGROUND_WHITE : foregroundColor,
              ),
            ),
          ],
        ),
      ),
    );
  }
}

class Title extends StatelessWidget {
  final Station station;

  Title({
    required this.station
  });

  @override
  Widget build(BuildContext context) {
    return Text(
      station.name,
      style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
        fontSize: 22,
        fontWeight: MediaQuery.of(context).boldText ? FontWeight.w500 : FontWeight.w300,
        // fontStyle: items[1] == "ONI" ? FontStyle.italic : FontStyle.normal,
      ),
      textAlign: TextAlign.center,
    );
  }
}

class Time extends StatelessWidget {
  final Station station;

  Time({
    required this.station,
  });

  @override
  Widget build(BuildContext context) {
    if (station.arrival == null) {
      // Plecare
      return DepartureTime(
        station: station,
        firstStation: true,
      );
    }

    if (station.departure == null) {
      // Sosire
      return ArrivalTime(
        station: station,
        finalStation: true,
      );
    }

    return Row(
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Text(
          "→",
          style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
            fontSize: 22,
          ),
        ),
        Container(width: 2,),
        ArrivalTime(station: station,),
        Expanded(child: Container(),),
        StopTime(station: station,),
        Expanded(child: Container(),),
        DepartureTime(station: station,),
        Container(width: 2,),
        Text(
          "→",
          style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
            fontSize: 22,
          ),
        ),
      ],
    );
  }
}

class ArrivalTime extends StatelessWidget {
  final Station station;
  final bool finalStation;

  ArrivalTime({
    required this.station,
    this.finalStation = false,
  });

  @override
  Widget build(BuildContext context) {
    if (finalStation) {
      return Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          Text(
            "→",
            style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
              fontSize: 22,
            ),
          ),
          Container(width: 2,),
          Text("sosire la "),
          ArrivalTime(station: station,),
          Expanded(child: Container(),),
        ],
      );
    }
    else {
      final delay = station.arrival!.status?.delay ?? 0;
      final time = DateTime.parse(station.arrival!.scheduleTime).toLocal();

      if (delay == 0) {
        return Text("${time.hour.toString().padLeft(2, "0")}:${time.minute.toString().padLeft(2, "0")}");
      }
      else if (delay > 0) {
        final oldDate = time;
        final newDate = oldDate.add(Duration(minutes: delay));

        return Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text(
              "${oldDate.hour.toString().padLeft(2, '0')}:${oldDate.minute.toString().padLeft(2, '0')}",
              style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                decoration: TextDecoration.lineThrough,
              ),
            ),
            Text(
              "${newDate.hour.toString().padLeft(2, '0')}:${newDate.minute.toString().padLeft(2, '0')}",
              style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                color: CupertinoColors.destructiveRed,
              ),
            ),
          ],
        );
      }
      else {
        final oldDate = time;
        final newDate = oldDate.add(Duration(minutes: delay));

        return Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text(
              "${oldDate.hour.toString().padLeft(2, '0')}:${oldDate.minute.toString().padLeft(2, '0')}",
              style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                decoration: TextDecoration.lineThrough,
              ),
            ),
            Text(
              "${newDate.hour.toString().padLeft(2, '0')}:${newDate.minute.toString().padLeft(2, '0')}",
              style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                color: CupertinoColors.activeGreen,
              ),
            ),
          ],
        );
      }
    }
  }
}

class StopTime extends StatelessWidget {
  final Station station;

  StopTime({
    required this.station,
  });

  @override
  Widget build(BuildContext context) {
    final stopsFor = station.stoppingTime!;
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Text(
          "staționează pentru",
          textAlign: TextAlign.center,
        ),
        Builder(
          builder: (context) {
            int stopsForInt = stopsFor;
            bool minutes = false;
            if (stopsForInt >= 60) {
              stopsForInt ~/= 60;
              minutes = true;
            }
            if (stopsForInt == 1) {
              return Text(
                minutes ? '1 minut' : '1 secundă',
                textAlign: TextAlign.center,
              );
            }
            else if (stopsForInt < 20) {
              return Text(
                '$stopsForInt ' + (minutes ?  'minute' : 'seconde'),
                textAlign: TextAlign.center,
              );
            }
            else {
              return Text(
                '$stopsForInt de ' + (minutes ? 'minute' : 'secunde'),
                textAlign: TextAlign.center,
              );
            }
          },
        )
      ],
    );
  }
}

class DepartureTime extends StatelessWidget {
  final Station station;
  final bool firstStation;

  DepartureTime({
    required this.station,
    this.firstStation = false,
  });

  @override
  Widget build(BuildContext context) {
    if (firstStation) {
      return Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          Expanded(child: Container(),),
          Text("plecare la "),
          DepartureTime(station: station,),
          Container(width: 2,),
          Text(
            "→",
            style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
              fontSize: 22,
            ),
          ),
        ],
      );
    }
    else {
      final delay = station.departure!.status?.delay ?? 0;
      final time = DateTime.parse(station.departure!.scheduleTime).toLocal();

      if (delay == 0) {
        return Text("${time.hour.toString().padLeft(2, "0")}:${time.minute.toString().padLeft(2, "0")}");
      }
      else if (delay > 0) {
        final oldDate = time;
        final newDate = oldDate.add(Duration(minutes: delay));

        return Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text(
              "${oldDate.hour.toString().padLeft(2, '0')}:${oldDate.minute.toString().padLeft(2, '0')}",
              style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                decoration: TextDecoration.lineThrough,
              ),
            ),
            Text(
              "${newDate.hour.toString().padLeft(2, '0')}:${newDate.minute.toString().padLeft(2, '0')}",
              style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                color: CupertinoColors.destructiveRed,
              ),
            ),
          ],
        );
      }
      else {
        final oldDate = time;
        final newDate = oldDate.add(Duration(minutes: delay));

        return Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text(
              "${oldDate.hour.toString().padLeft(2, '0')}:${oldDate.minute.toString().padLeft(2, '0')}",
              style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                decoration: TextDecoration.lineThrough,
              ),
            ),
            Text(
              "${newDate.hour.toString().padLeft(2, '0')}:${newDate.minute.toString().padLeft(2, '0')}",
              style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
                color: CupertinoColors.activeGreen,
              ),
            ),
          ],
        );
      }
    }
  }
}

class Delay extends StatelessWidget {
  final Station station;

  Delay({
    required this.station,
  });

  @override
  Widget build(BuildContext context) {
    if (station.arrival?.status == null && station.departure?.status == null) {
      return Container();
    }
    var delay = station.arrival?.status?.delay;
    if (station.departure?.status?.real == true) {
      delay = station.departure?.status?.delay;
    }

    if (delay == 0 || delay == null) return Container();
    else if (delay > 0) {
      return Text(
        "$delay ${delay == 1 ? 'minut' : 'minute'} întârziere",
        style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
          color: CupertinoColors.destructiveRed,
          fontSize: 14,
          fontStyle: FontStyle.italic,
        ),
      );
    }
    else if (delay < 0) {
      return Text(
        "${-delay} ${delay == -1 ? 'minut' : 'minute'} mai devreme",
        style: CupertinoTheme.of(context).textTheme.textStyle.copyWith(
          color: CupertinoColors.activeGreen,
          fontSize: 14,
          fontStyle: FontStyle.italic,
        ),
      );
    }

    return Container();
  }
}
